<?php

namespace Bixo\Group\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Group\Forms\Group as GroupForm;
use Bixo\Group\Http\Requests\GroupResourceRequest;
use Bixo\Group\Http\Resources\GroupResource;
use Bixo\Group\Http\Resources\GroupsCollection;
use Bixo\Group\Models\Group;
use Bixo\Group\Actions\GroupAction;
use Bixo\Group\Actions\GroupActions;


/**
 * Resource controller class for group.
 */
class GroupResourceController extends BaseController
{

    /**
     * Initialize group resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = GroupForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.group.modules'), 'group', guard_url('group'));
            return $next($request);
        });
    }

    /**
     * Display a list of group.
     *
     * @return Response
     */
    public function index(GroupResourceRequest $request)
    {
        $request = $request->all();
        $page = GroupActions::run('paginate', $request);

        $data = new GroupsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('group::group.names'))
            ->view('group::group.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display group.
     *
     * @param Request $request
     * @param Model   $group
     *
     * @return Response
     */
    public function show(GroupResourceRequest $request, Group $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new GroupResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('group::group.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('group::group.show')
            ->output();
    }

    /**
     * Show the form for creating a new group.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(GroupResourceRequest $request, Group $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new GroupResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('group::group.name'))
            ->view('group::group.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new group.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(GroupResourceRequest $request, Group $model)
    {
        try {
            $request = $request->all();
            $model = GroupAction::run('store', $model, $request);
            $data = new GroupResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('group::group.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('group/group/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/group/group'))
                ->redirect();
        }

    }

    /**
     * Show group for editing.
     *
     * @param Request $request
     * @param Model   $group
     *
     * @return Response
     */
    public function edit(GroupResourceRequest $request, Group $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new GroupResource($model);
        // return view('group::group.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('group::group.name'))
            ->view('group::group.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the group.
     *
     * @param Request $request
     * @param Model   $group
     *
     * @return Response
     */
    public function update(GroupResourceRequest $request, Group $model)
    {
        try {
            $request = $request->all();
            $model = GroupAction::run('update', $model, $request);
            $data = new GroupResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('group::group.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('group/group/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('group/group/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the group.
     *
     * @param Model   $group
     *
     * @return Response
     */
    public function destroy(GroupResourceRequest $request, Group $model)
    {
        try {

            $request = $request->all();
            $model = GroupAction::run('destroy', $model, $request);
            $data = new GroupResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('group::group.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('group/group/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('group/group/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
